package net.wanji.opt.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.mapper.GreenwaveHistMapper;
import net.wanji.databus.dto.EventInfoTrafficStatusDTO;
import net.wanji.databus.po.EventInfoTrafficStatusPO;
import net.wanji.databus.vo.EventInfoTrafficStatusVO;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.common.enums.EventInfoTypeEnum;
import net.wanji.opt.dao.mapper.HoloEventMapper;
import net.wanji.opt.dao.mapper.base.BaseRidInfoMapper;
import net.wanji.opt.entity.EventAlarmInfo;
import net.wanji.opt.po.base.BaseRidInfo;
import net.wanji.opt.po.base.EventStatisticPo;
import net.wanji.opt.po.trend.EventInfoSimplePo;
import net.wanji.opt.service.EventService;
import org.joda.time.DateTime;
import org.joda.time.Minutes;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 *
 */
@Slf4j
@Service
public class EventServiceImpl implements EventService {
	@Resource
	HoloEventMapper holoEventMapper;

	@Resource
	BaseRidInfoMapper baseRidInfoMapper;

	@Resource
	GreenwaveHistMapper greenwaveHistMapper;


	@Override
	public EventStatisticPo findCrossEventCount(String crossId, String type, String startTime, String endTime) throws DubboProviderException {
		Map<String, Object> params = new HashMap<>();
		params.put("crossId", crossId);
		params.put("startDate", startTime);
		params.put("endDate", endTime);
		params.put("eventType", type);

		EventStatisticPo po = holoEventMapper.findCrossEventCount(params);

		if (po != null) {
			String dirs = po.getDirs();
			Map<String, Long> countMap = null;
			List<Map<String, Object>> list = new ArrayList<>();
			if (Objects.nonNull(dirs)) {
				String[] dirList = dirs.split("@");
				List<String> allDirList = new ArrayList<>();
				for (String dirArr : dirList) {
					if (!dirArr.startsWith("[") && !dirArr.endsWith("]")){
						//过滤不合规格式数据
						continue;
					}
					List<String> jsonArray = JSONArray.parseArray(dirArr, String.class);
					if (jsonArray.size()>0) {
						allDirList.addAll(jsonArray.stream().map(o -> (Integer.valueOf(o).toString())).distinct().collect(Collectors.toList()));
					}
				}
				//统计每个方向发生的次数
				countMap = allDirList.stream().collect(Collectors.groupingBy(String::valueOf, Collectors.counting()));
				countMap.entrySet().forEach(o -> {
					Map<String, Object> map = new HashMap<>();
					map.put("dir", o.getKey());
					map.put("count", o.getValue());
					list.add(map);
				});
			}
			po.setDirEventCount(countMap);
			po.setDirEventCountList(list);


			QueryWrapper<BaseRidInfo> queryWrapper = new QueryWrapper<>();
			queryWrapper.eq("end_cross_id", crossId);

			List<BaseRidInfo> ridList = baseRidInfoMapper.selectList(queryWrapper);

			Map<String, List<String>> group = ridList.stream().collect(Collectors.groupingBy(o -> o.getInDir().toString(), Collectors.mapping(BaseRidInfo::getWkt, Collectors.toList())));

			//进口道方向wkt
			Map<String, String> dirWkt = new HashMap<>();
			group.forEach((k, v) -> {
				dirWkt.put(k, v.get(0));
			});
			po.setDirs(null);
			po.setDirWkt(dirWkt);
		}
		return po;
	}

	@Override
	public List<Map<String, Object>> findEventDistribute(String crossId, String startTime, String endTime) {
		Map<String, Object> params = new HashMap<>();
		params.put("crossId", crossId);
		params.put("startDate", startTime);
		params.put("endDate", endTime);

		List<Map<String, Object>> retList = new ArrayList<>();

		DateTime start = DateTime.parse(startTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
		DateTime end = DateTime.parse(endTime, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER));
		//存放时段
		Set<String> sortedSet = getTimeScopeList(start, end, EsDateIndexUtil.YMD_HM_FORMATTER);

		List<EventInfoSimplePo> list = holoEventMapper.findEventDistribute(params);
		//每5分组分组
		Map<String, List<EventInfoSimplePo>> group = list.stream().collect(Collectors.groupingBy(o -> o.getTimeAxisStart()));

		List<EventInfoSimplePo> tmpList = new ArrayList<>();
		List<String> dataserList = new ArrayList<>();

		group.forEach((time, valList1) -> {
			//5分组内数据按类型分组
			Map<String, List<EventInfoSimplePo>> groupByType = valList1.stream().collect(Collectors.groupingBy(o -> o.getEventType()));

			//List<EventInfoSimplePo> tmpList = new ArrayList<>();

			groupByType.forEach((type, valList2) -> {
				EventInfoSimplePo tmp = null;
				for (EventInfoSimplePo po : valList2) {
					po.setDate(net.wanji.common.utils.tool.DateUtil.formatDate(po.getStartTime(), "M/d"));
					//合并时间重叠事件
					if (tmp != null) {
						Date agoStartTime = tmp.getStartTime();
						Date agoEndTime = tmp.getEndTime();
						Date currentStartTime = po.getStartTime();
						Date currentEndTime = po.getEndTime();
						if (currentStartTime.before(agoEndTime)) {
							int duration = Seconds.secondsBetween(new DateTime(agoStartTime.getTime()), new DateTime(currentEndTime.getTime())).getSeconds();
							po.setDuration(duration);
							po.setStartTime(agoStartTime);
							po.setEndTime(currentEndTime);
							po.setTimeAxisStart(tmp.getTimeAxisStart());
							po.setCount(po.getCount() + 1);
							//清理合并前的记录
							tmpList.remove(tmp);
						}
					}
					//==============周期数据时间范围============================================//
					DateTime startAxis = new DateTime(DateTime.parse(po.getTimeAxisStart(), DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")));
					DateTime endAxis = new DateTime(po.getEndTime());
					int diffNum = Minutes.minutesBetween(startAxis, endAxis).getMinutes();

					List<String> dataTimeList = po.getDataTimeList();
					for (int i = 0; i <= diffNum; i++) {
						DateTime axisDateTime = startAxis.plusMinutes(i * 1);
						int m = axisDateTime.getMinuteOfHour();
						if (m % 5 == 0 && axisDateTime.isBefore(endAxis)) {
							dataTimeList.add(axisDateTime.toString("yyyy-MM-dd HH:mm:ss"));
						}
					}
					po.setTimeAxisEnd(endAxis.toString("yyyy-MM-dd HH:mm:ss"));
					po.setDataTimeList(dataTimeList);
					//======================================================================//
					tmpList.add(po);
					tmp = po;
				}
			});
		});
		List<EventInfoSimplePo> sortList = tmpList.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
		sortList.forEach(po -> {
			String axis = DateUtil.formatDate(po.getStartTime(), "M/d") + "," + DateUtil.formatDate(po.getStartTime(), "HH:mm:ss") + "," + po.getEventType();
			dataserList.add(axis);
		});
		Map<String, Object> map = new HashMap<>();
		map.put("statusByDateList", sortList);
		map.put("dataser", dataserList);
		retList.add(map);
//        Set<String> timeList = group.keySet();
		//补充缺少时段数据，保留时段字段默认值
//        for (String timeSec : sortedSet) {
//            if (!timeList.contains(timeSec)) {
//                Map<String, Object> map = new HashMap<>();
//                map.put("timeAxis", timeSec);
//                map.put("statusByDateList", new ArrayList<>());
//                retList.add(map);
//            }
//        }

//        List<Map<String, Object>> sortList = retList.stream().sorted(Comparator.comparing(o -> o.get("timeAxis").toString())).collect(Collectors.toList());

		return retList;
	}

	@Override
	public EventInfoTrafficStatusVO trafficStatus(EventInfoTrafficStatusDTO eventInfoTrafficStatusDTO) throws DubboProviderException {
		EventInfoTrafficStatusVO eventInfoTrafficStatusVO = new EventInfoTrafficStatusVO();
		String eventType = eventInfoTrafficStatusDTO.getEventType();
		Integer queryType = eventInfoTrafficStatusDTO.getQueryType();
		List<EventInfoTrafficStatusPO> eventInfoTrafficStatusPOS = null;
		if (ObjectUtil.equals(1, queryType)) {
			//路口
			//获取type
			String type = EventInfoTypeEnum.getEventInfoTypeEnum(Integer.valueOf(eventType)).getEventType();
			eventInfoTrafficStatusDTO.setEventType(type);
			eventInfoTrafficStatusPOS = holoEventMapper.trafficStatus(eventInfoTrafficStatusDTO);
		} else {
			//干线
			List<Integer> statusList = new ArrayList<>();
			if (ObjectUtil.equals("6", eventType)) {
				//干线缓行
				statusList.add(2);
			} else {
				//干线拥堵
				statusList.add(3);
				statusList.add(4);
			}
			List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectByIdAndType(Integer.valueOf(eventInfoTrafficStatusDTO.getId()), statusList);
			if (ObjectUtil.isNotEmpty(greenwaveHistPOS)) {
				eventInfoTrafficStatusPOS = new ArrayList<EventInfoTrafficStatusPO>();
				Map<Integer, List<GreenwaveHistPO>> collect = greenwaveHistPOS.stream().collect(Collectors.groupingBy(x -> x.getStartTime().getHours()));
				for (Map.Entry<Integer, List<GreenwaveHistPO>> entry : collect.entrySet()) {
					eventInfoTrafficStatusPOS.add(handerEntry(entry));
				}
			}
		}

		//获取当前时间的时间轴
		Set<String> timeScopeList = getTimeScopeList(DateTime.now().withTimeAtStartOfDay(), new DateTime(), EsDateIndexUtil.H_FORMATTER);
		if (ObjectUtil.isNotEmpty(timeScopeList)) {
			ArrayList<String> arrayList = timeScopeList.stream().collect(Collectors.toCollection(ArrayList::new));
			List<String> collect = arrayList.stream().sorted(Comparator.comparing(time -> Integer.valueOf(time.split(":")[0]))).collect(Collectors.toList());
			eventInfoTrafficStatusVO.setTimeList(collect);
			//补充全部数据
			if (ObjectUtil.isEmpty(eventInfoTrafficStatusPOS)) {
				int size = timeScopeList.size();
				List<Integer> list = getDefaultList(size);
				eventInfoTrafficStatusVO.setEventCount(list);
				eventInfoTrafficStatusVO.setDurationTime(list);
			} else {
				List<String> timeList = eventInfoTrafficStatusPOS.stream().map(EventInfoTrafficStatusPO::getTimeAxisStart).collect(Collectors.toList());
				//补充缺少时段数据
				for (String s : timeScopeList) {
					if (!timeList.contains(s)) {
						EventInfoTrafficStatusPO eventInfoTrafficStatusPO = new EventInfoTrafficStatusPO();
						eventInfoTrafficStatusPO.setDuration(0);
						eventInfoTrafficStatusPO.setCount(0);
						eventInfoTrafficStatusPO.setTimeAxisStart(s);
						eventInfoTrafficStatusPOS.add(eventInfoTrafficStatusPO);
					}
				}
				List<EventInfoTrafficStatusPO> trafficStatusPOS = eventInfoTrafficStatusPOS.stream().sorted(Comparator.comparing(time -> Integer.valueOf(time.getTimeAxisStart().split(":")[0]))).collect(Collectors.toList());
				List<Integer> durationList = trafficStatusPOS.stream().map(x -> x.getDuration()).collect(Collectors.toList());
				List<Integer> countList = trafficStatusPOS.stream().map(x -> x.getCount()).collect(Collectors.toList());
				eventInfoTrafficStatusVO.setEventCount(countList);
				eventInfoTrafficStatusVO.setDurationTime(durationList);
			}
		}
		return eventInfoTrafficStatusVO;
	}

	/**
	 * @return net.wanji.databus.po.EventInfoTrafficStatusPO
	 * @Description 干线数据处理
	 * @Param [entry]
	 **/
	private EventInfoTrafficStatusPO handerEntry(Map.Entry<Integer, List<GreenwaveHistPO>> entry) {
		EventInfoTrafficStatusPO eventInfoTrafficStatusPO = new EventInfoTrafficStatusPO();
		Integer hour = entry.getKey();
		String timeAxisStart = null;
		if (hour <= 9) {
			timeAxisStart = "0" + entry.getKey() + ":00";
		} else {
			timeAxisStart = entry.getKey() + ":00";
		}
		int count = 0;
		List<GreenwaveHistPO> value = entry.getValue();
		for (int i = 0; i < value.size(); i++) {
			if (i == 0) {
				count++;
			} else {
				if (value.get(i).getStartTime().getTime() - value.get(i - 1).getStartTime().getTime() >= 300) {
					count++;
				}
			}
		}

		int duration = value.size() * 300;
		eventInfoTrafficStatusPO.setDuration(duration);
		eventInfoTrafficStatusPO.setCount(count);
		eventInfoTrafficStatusPO.setTimeAxisStart(timeAxisStart);
		return eventInfoTrafficStatusPO;
	}

	/**
	 * @return java.util.List<java.lang.Integer>
	 * @Description 根据集合长度，获取默认0补充的list
	 * @Param [size]  长度
	 **/
	private List<Integer> getDefaultList(int size) {
		List<Integer> list = new ArrayList<>();
		for (int i = 0; i < size; i++) {
			list.add(0);
		}
		return list;
	}

	@Override
	public List<EventAlarmInfo> findEventAlarmInfo(String realtimeType, String eventType, String name) throws DubboProviderException {
		//获取数据库数据
		List<EventAlarmInfo> eventAlarmInfo = holoEventMapper.findEventAlarmInfo(realtimeType, eventType, name);

		return eventAlarmInfo;
	}

	@Override
	public List<EventAlarmInfo> findNotFinishAlarmInfo() throws DubboProviderException {
		List<EventAlarmInfo> list = holoEventMapper.findNotFinishAlarmInfo();
		for (EventAlarmInfo info : list) {
			String dir = info.getDir();
			String turn = info.getTurn();

			if (Objects.nonNull(dir)) {
				List<Integer> dirJsonArray = JSONArray.parseArray(dir, Integer.class);
				//dirJsonArray = dirJsonArray.stream().distinct().collect(Collectors.toList());

				StringBuilder stringBuilder = new StringBuilder();
				stringBuilder.append(info.getCurrentAlgo() + "发生方向：");
				if (Objects.nonNull(turn)) {
					//有方向、有转向
					List<Integer> turnJsonArray = JSONArray.parseArray(turn, Integer.class);
					//turnJsonArray = turnJsonArray.stream().distinct().collect(Collectors.toList());
					for (int i = 0; i < dirJsonArray.size(); i++) {
						String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.valueOf(dirJsonArray.get(i))) + "进口";
						if (Objects.equals(3,info.getStatus())){
                            dirName = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.valueOf(dirJsonArray.get(i))) + "出口";
                        }
						String turnName = BaseEnum.TurnTypeEnum.getNameByCode(turnJsonArray.get(i));
						String desc = dirName + turnName;
//                        if (stringBuilder.indexOf(desc)>0){
//                            continue;
//                        }
						if (i != dirJsonArray.size() - 1) {
							stringBuilder.append(desc);
							stringBuilder.append("；");
						} else {
							stringBuilder.append(desc);
						}
					}
				} else {
					//只有方向
					for (int i = 0; i < dirJsonArray.size(); i++) {
						String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.valueOf(dirJsonArray.get(i))) + "进口";
                        if (Objects.equals(3,info.getStatus())){
                            dirName = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.valueOf(dirJsonArray.get(i))) + "出口";
                        }
//                        if (stringBuilder.indexOf(dirName)>0){
//                            continue;
//                        }
						if (i != dirJsonArray.size() - 1) {
							stringBuilder.append(dirName);
							stringBuilder.append("；");
						} else {
							stringBuilder.append(dirName);
						}
					}
				}
				info.setDirDesc(stringBuilder.toString());
			}

		}
		return list;
	}


	public static Set<String> getTimeScopeList(DateTime start, DateTime end, String dateFormat) {
		Set<String> indexList = new HashSet<>();
		int diffNum = Minutes.minutesBetween(start, end).getMinutes();
		int unitTime = 5;
		for (int i = 0; i <= diffNum; i = i + unitTime) {
			String format = start.plusMinutes(i).toString(dateFormat);
			indexList.add(format);
		}
		return indexList;
	}
}